---
title: Install Chaos Mesh using Helm
---

import Tabs from '@theme/Tabs'

import TabItem from '@theme/TabItem'

import PickVersion from '@site/src/components/PickVersion'

import PickHelmVersion from '@site/src/components/PickHelmVersion'

import VerifyInstallation from './common/\_verify-installation.md'

import QuickRun from './common/\_quick-run.md'

This document describes how to install Chaos Mesh in the production environment.

## Prerequisites

Before installing Chaos Mesh, make sure that you have installed [Helm](https://helm.sh/docs/intro/install/) in your environment.

To check whether Helm is installed or not, execute the following command:

```bash
helm version
```

The expected output is as follows:

```bash
version.BuildInfo{Version:"v3.5.4", GitCommit:"1b5edb69df3d3a08df77c9902dc17af864ff05d1", GitTreeState:"dirty", GoVersion: "go1.16.3"}
```

If your actual output is similar to the expected output with `Version`, `GitCommit`, `GitTreeState`, and `GoVersion`, it means that Helm is installed successfully.

:::note

In this document, Helm v3 is used in commands to make operations on Chaos Mesh. If Helm v2 is used in your environment, refer to [Migrating Helm v2 to v3](https://helm.sh/docs/topics/v2_v3_migration/) or modify the Helm version to the v2 format.

:::

## Install Chaos Mesh using Helm

### Step 1: Add Chaos Mesh repository

Add the Chaos Mesh repository to the Helm repository:

```bash
helm repo add chaos-mesh https://charts.chaos-mesh.org
```

### Step 2: View the installable versions of Chaos Mesh

To see charts that can be installed, execute the following command:

```bash
helm search repo chaos-mesh
```

:::note

The above command will output the latest release of chart. If you want to install a historical version, execute the following command to view all released versions:

```bash
helm search repo chaos-mesh -l
```

:::

After the above command is completed, you can start installing Chaos Mesh.

### Step 3: Create the namespace to install Chaos Mesh

It is recommended to install Chaos Mesh under the `chaos-mesh` namespace, or you can specify any namespace to install Chaos Mesh:

```bash
kubectl create ns chaos-mesh
```

### Step 4: Install Chaos Mesh in different environments

:::note

When installing Chaos Mesh on Kubernetes v1.15(or an earlier version), you need to manually install CRD. For more information, see [FAQ](./faqs.md#failed-to-install-chaos-mesh-with-the-message-no-matches-for-kind-customresourcedefinition-in-version-apiextensionsk8siov1).

:::

As the daemons of different container runtimes listen on different socket paths, you need to set the appropriate values during installation. You can run the following installation commands according to different environments.

<!-- prettier-ignore -->
<Tabs defaultValue="docker" values={[
  {label: 'Docker', value: 'docker'},
  {label: 'Containerd', value: 'containerd'},
  {label: 'K3s', value: 'k3s'},
  {label: 'MicroK8s', value: 'microk8s'},
  {label: 'CRI-O', value: 'cri-o'}
]}>
  <TabItem value="docker">
    <PickHelmVersion>{`\# Default to /var/run/docker.sock\nhelm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --version latest`}</PickHelmVersion>
  </TabItem>
  <TabItem value="containerd">
    <PickHelmVersion>{`helm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --set chaosDaemon.runtime=containerd --set chaosDaemon.socketPath=/run/containerd/containerd.sock --version latest`}</PickHelmVersion>
  </TabItem>
  <TabItem value="k3s">
    <PickHelmVersion>{`helm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --set chaosDaemon.runtime=containerd --set chaosDaemon.socketPath=/run/k3s/containerd/containerd.sock --version latest`}</PickHelmVersion>
  </TabItem>
  <TabItem value="microk8s">
    <PickHelmVersion>{`helm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --set chaosDaemon.runtime=containerd --set chaosDaemon.socketPath=/var/snap/microk8s/common/run/containerd.sock --version latest`}</PickHelmVersion>
  </TabItem>
  <TabItem value="cri-o">
    <PickHelmVersion>{`helm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --set chaosDaemon.runtime=crio --set chaosDaemon.socketPath=/var/run/crio/crio.sock --version latest`}</PickHelmVersion>
  </TabItem>
</Tabs>

:::info

To install a specific version of Chaos Mesh, add the `--version x.y.z` parameter after `helm install`. For example, `helm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --version 2.1.0`.

:::

:::tip

To ensure high availability, Chaos Mesh turns on `leader-election` feature by default. If you do not need to use this feature, you can disable it manually with `--set controllerManager.leaderElection.enabled=false`.

> If version `<2.6.1`, you still need to set `--set controllerManager.replicaCount=1` to reduce the controller manager to one replica.

:::

## Verify the installation

<VerifyInstallation />

## Run Chaos experiments

<QuickRun />

## Upgrade Chaos Mesh

To upgrade Chaos Mesh, execute the following command:

```bash
helm upgrade chaos-mesh chaos-mesh/chaos-mesh
```

:::info

To upgrade to a specific version of Chaos Mesh, add the `--version x.y.z` parameter after `helm upgrade`. For example, `helm upgrade chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --version 2.1.0`.

:::

:::note

If you have upgraded Chaos Mesh in a non-Docker environment, you will need to add the appropriate parameters as described in [Step 4: Install Chaos Mesh in different environments](#step-4-install-chaos-mesh-in-different-environments).

:::

To modify the configuration, set different values according to your need. For example, execute the following command to upgrade and uninstall `chaos-dashboard`:

<PickHelmVersion>{`helm upgrade chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --version latest --set dashboard.create=false`}</PickHelmVersion>

:::note

For more values and their usages, refer to [all values](https://github.com/chaos-mesh/chaos-mesh/blob/master/helm/chaos-mesh/values.yaml).

:::

:::caution

Currently, the latest CustomResourceDefinition (CRD) is not applied during the Helm upgrading, which might cause errors. To avoid this situation, you can apply the latest CRD manually:

<PickVersion>
{`curl -sSL https://github.com/chaos-mesh/chaos-mesh/raw/refs/heads/master/manifests/crd.yaml | kubectl create -f -`}
</PickVersion>

:::

## Uninstall Chaos Mesh

To uninstall Chaos Mesh, execute the following command:

```bash
helm uninstall chaos-mesh -n chaos-mesh
```

## FAQs

### How can I install the latest version of Chaos Mesh?

The `helm/chaos-mesh/values.yaml` file defines the image of the latest version (the master branch). To install the latest version of Chaos Mesh, execute the following command:

```bash
# Clone repository
git clone https://github.com/chaos-mesh/chaos-mesh.git
cd chaos-mesh

helm install chaos-mesh helm/chaos-mesh -n=chaos-mesh
```

### How can I disable the safe mode?

Safe mode allows you to disable authentication to the Chaos Mesh dashboard, and should only be used for non-production deployments. Safe mode is **enabled** by default. To disable the safe mode, specify `dashboard.securityMode` as `false` during the installation or upgrade:

<PickHelmVersion>
{`helm install chaos-mesh chaos-mesh/chaos-mesh -n=chaos-mesh --set dashboard.securityMode=false --version latest`}
</PickHelmVersion>

### How can I persist Chaos Dashboard data

Chaos Dashboard uses `SQLite` as the default database engine. If [PV (Persistent Volumes)](https://kubernetes.io/docs/concepts/storage/persistent-volumes/) is disabled, the data of Chaos Dashboard will be lost after a reboot occurs. To avoid data loss, you can refer to the [Persistence of Chaos Dashboard Data](persistence-dashboard.md) document to enable PV for Chaos Dashboard or set `MySQL` and `PostgreSQL` as the database engine.
